SystemOrganization addCategory: #VisuAule!
SystemOrganization addCategory: #'VisuAule-Objects'!
SystemOrganization addCategory: #'VisuAule-Objects-Tests'!
SystemOrganization addCategory: #'VisuAule-Example'!

TestCase subclass: #VABookAndPersonTest
	instanceVariableNames: 'book ralph eric joe'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects-Tests'!

!VABookAndPersonTest methodsFor: 'running' stamp: 'CamiloGomez 7/6/2012 18:39'!
setUp
	
	book := VABook new name: 'design patterns'.
	ralph := VAPerson new name: 'Ralph Johnson'.
	eric := VAPerson new name: 'Eric Gamma'.
	book addAuthor: ralph; addAuthor: eric.
	
	joe := VAReader new.
	joe readBook: book.! !

!VABookAndPersonTest methodsFor: 'testing scenarios' stamp: 'CamiloGomez 7/5/2012 12:12'!
testPerson

	"The scenario we are here simulating is (person visualize)"
	
	| view |
	
	view := ROView new.
	
	ralph visualizeOn: view.
	eric visualizeOn: view.
	
	"view open"
	
	self assert: view elements size = 2! !

!VABookAndPersonTest methodsFor: 'testing scenarios' stamp: 'CamiloGomez 7/5/2012 12:12'!
testVisualizingBooksAndPersons

	"The scenario we are here simulating is (book new visualize)"
	
	| view |
		
	view := ROView new.
	
	self assert: view elements isEmpty.
	book visualizeOn: view.
	"view open"
	self assert: view elements size = 5.
	self assert: (view elementFromModel: book) notNil.
	self assert: (view elementFromModel: ralph) notNil.
	self assert: (view elementFromModel: eric) notNil.
	self assert: (view elementFromModel: joe) isNil.! !

TestCase subclass: #VAMovieTest
	instanceVariableNames: 'lordOfTheRings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects-Tests'!

!VAMovieTest methodsFor: 'running' stamp: 'CamiloGomez 7/6/2012 16:52'!
setUp

	lordOfTheRings := VAMovie new.
	
	lordOfTheRings title: 'The Lord Of The Rings: The Fellowship Of The Ring'. 
	lordOfTheRings director: (VAPerson new name: 'Peter Jackson').
	lordOfTheRings duration: '3 hours'.
	lordOfTheRings country: 'New Zealand'.
	lordOfTheRings genre: 'Fantasy'.
	! !

!VAMovieTest methodsFor: 'tests' stamp: 'CamiloGomez 7/6/2012 16:58'!
testVisualizeMovie

	| view |
	
	view := ROView new.
	
	self assert: view elements isEmpty.
	
	lordOfTheRings visualizeOn: view.
	
	self assert: view elements size = 3.
	self assert: (view elementFromModel: lordOfTheRings) notNil.
	! !

TestCase subclass: #VAVisualizingPointTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects-Tests'!

!VAVisualizingPointTest methodsFor: 'testing scenarios' stamp: 'CamiloGomez 7/6/2012 16:37'!
testVisualizingPoint

	"The scenario we are here simulating is (2@4) visualize"
	
	| view |
	view := ROView new.
	
	self assert: view elements isEmpty.
	(2 @ 4) visualizeOn: view.
	(10 @ 10) visualizeOn: view.
	"view open."
	self assert: view elements size = 2.! !

!Browser methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/6/2012 12:23'!
interactionFor: element on: view

	super interactionFor: element on: view.
	
	element on: ROMouseLeftClick do: [ :event | Browser open].! !

!Number methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 6/27/2012 15:38'!
shouldBeAddedAsElement
	^ false! !

!WriteStream methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/9/2012 16:45'!
shouldBeAddedAsElement
	^true! !

!UndefinedObject methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 6/27/2012 15:50'!
shouldBeAddedAsElement
	^ false! !

!UndefinedObject methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/17/2012 17:15'!
shouldBeAddedAsElementOnEnter
	^ false! !

!Mutex methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/9/2012 16:37'!
shouldBeAddedAsElement
	^true! !

!ROEaselMorphic methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/6/2012 17:12'!
interactionFor: element on: view

	super interactionFor: element on: view.
	
	element on: ROMouseLeftClick do: [ :event | ROEaselMorphic new open]. ! !

!SystemOrganizer methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/9/2012 16:37'!
shouldBeAddedAsElement
	^true! !

!SystemOrganizer methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/17/2012 17:11'!
shouldBeAddedAsElementOnEnter 
	^false! !

!Boolean methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/17/2012 17:23'!
shouldBeAddedAsElement 
	^ false! !

!ThreadSafeTranscript methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/6/2012 15:55'!
interactionFor: element on: view

	super interactionFor: element on: view.
	
	element on: ROMouseLeftClick do: [ :event | Transcript open].! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/23/2012 12:42'!
addElement: el of: elementForSelf on: view

	 (el model) interactionFor: el on: view.
	 view add: el.
	 view add: (ROEdge lineFrom: elementForSelf to: el)! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/24/2012 18:00'!
addReferencedObjectsOf: elementForSelf on: view

	 self class allInstVarNames do: [ :varName |
		| el object |
		object := self instVarNamed: varName.
		object shouldBeAddedAsElement
			ifTrue: 
				[object isCollection
					ifTrue: [self handleCollectionIn: object of: elementForSelf on: view]
					ifFalse: [el := (elementForSelf model) roassalElementFor: object.
				 			(elementForSelf model) addElement: el of: elementForSelf on: view.]]] ! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/24/2012 15:59'!
elementsNotAdded

	| otherElements elementObject newElement varNames varNamesElement otherElementsElement bothColumns |
	
	bothColumns := OrderedCollection new.
	varNamesElement := ROElement new.
	otherElementsElement := ROElement new.
	newElement := ROElement new.
	varNames := OrderedCollection new.
	otherElements := OrderedCollection new.
	
	self class allInstVarNames do: [ :varName |
		| el object |
		object := self instVarNamed: varName.
		elementObject := self roassalOtherElementFor: object.
		object shouldBeAddedAsElementOnEnter
		ifTrue: [ 
			object notNil 
			ifTrue: [ 
				varNames add: ((ROElement on: varName) + ROLabel) .
				otherElements add: elementObject ]]].
	
	"self setThisLayout: ROVerticalLineLayout on: varNames.
	self setThisLayout: ROVerticalLineLayout on: otherElements."
	ROVerticalLineLayout on: varNames.
	ROVerticalLineLayout on: otherElements.
	
	varNamesElement addAll: varNames.
	otherElementsElement addAll: otherElements.
	
	bothColumns add: varNamesElement.
	bothColumns add: otherElementsElement.
	"self setThisLayout: ROHorizontalLineLayout on: bothColumns."
	ROHorizontalLineLayout on: bothColumns. 
	newElement addAll: bothColumns.
	
	newElement + ROBorder.
	newElement shapesDo: [ :d | d color: Color red].
	
	^ newElement! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/23/2012 12:14'!
handleCollectionIn: object of: elementForSelf on: view

	| el |
	
	(object size < 5)
	ifTrue: [object do: [ :element | 
					element notNil
					ifTrue: [el := (elementForSelf model) roassalElementFor: element.
				 	(elementForSelf model) addElement: el of: elementForSelf on: view.]]]
	ifFalse: [el := (elementForSelf model) roassalElementFor: object.
			(elementForSelf model) addElement: el of: elementForSelf on: view.
			]
 ! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/17/2012 17:30'!
interactionFor: elementForSelf on: view

	| otherElements elementObject |
	elementForSelf @ (ROMenuActivable new item: 'inspect element' action: #inspect;
									item: 'browse element class' action: #browse).

	otherElements := (elementForSelf model) elementsNotAdded.
	
	elementForSelf on: ROMouseEnter do: [ :event | 
			view add: otherElements.
			otherElements translateTo: ((elementForSelf position) + (0@20))].
		
	elementForSelf on: ROMouseLeave do: [ :event | 
			view remove: otherElements]! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/24/2012 16:44'!
roassalElementFor: object
	| element |
	
	element := ROElement on: object.
	element + (ROLabel new text: object class) + ROBorder.
	element translateTo: 5@5 .
	element @ ROLightlyHighlightable @ RODraggable.
	"element shapesDo: [ :d | d text: object class]."
	"element shapesDo: [ :d | d color: Color red]."
	
	^ element! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/24/2012 16:39'!
roassalOtherElementFor: object
	| element |
	
	element := ROElement on: object.
	element + ROLabel.
	element @ ROLightlyHighlightable @ RODraggable.
	"element shapesDo: [ :d | d color: Color red]."
	
	^ element! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/13/2012 17:42'!
setLayoutOn: view

	ROGridLayout new 
				translator: (ROSmoothLayoutTranslator new);
				applyOn: view elements.! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/9/2012 17:16'!
setThisLayout: layout on: elements

	layout new 
				translator: (ROSmoothLayoutTranslator new);
				applyOn: elements.
			
	"I don't know yet if this is gonna be necessary"! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/17/2012 17:23'!
shouldBeAddedAsElement
	^ true! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/17/2012 17:12'!
shouldBeAddedAsElementOnEnter
	^ true! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/6/2012 15:57'!
visualize

	| view obj |
	
	view := ROView new.
	
	view title: 'VisuAule'.
		
	self visualizeOn: view.
	
	view open.
! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/18/2012 16:20'!
visualizeOn: view  
	"We add the object"
	 | elementForSelf |
	elementForSelf := self roassalElementFor: self.
	self interactionFor: elementForSelf on: view.
	view add: elementForSelf.
	
	"We set the title to the window"
	view title: 'VisuAule'.
	
	"We add the referenced objects"
	self addReferencedObjectsOf: elementForSelf on: view.
	
	"We set a Layout, in this case, gridLayout"
	self setThisLayout: ROGridLayout on: (view elements). ! !

Object subclass: #VABook
	instanceVariableNames: 'authors title name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects'!

!VABook methodsFor: 'actions' stamp: 'CamiloGomez 6/27/2012 15:18'!
addAuthor: aPerson
	authors add: aPerson! !

!VABook methodsFor: 'initialize-release' stamp: 'CamiloGomez 6/27/2012 15:17'!
initialize
	super initialize.
	authors := OrderedCollection new.! !

!VABook methodsFor: 'accessing' stamp: 'CamiloGomez 6/27/2012 15:26'!
name: aString
	name := aString! !

Object subclass: #VAExample
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Example'!

!VAExample commentStamp: '<historical>' prior: 0!
List of examples!

!VAExample methodsFor: 'open tools' stamp: 'CamiloGomez 7/6/2012 17:11'!
allTools

	"self new allTools"

	| view |
	
	view := ROView new.
	
	Browser new visualizeOn: view.
	Transcript visualizeOn: view.
	ROEaselMorphic new visualizeOn: view.
	
	view open.! !

!VAExample methodsFor: 'open tools' stamp: 'CamiloGomez 7/6/2012 12:20'!
browser
	"self new browser"

	Browser new visualize.! !

!VAExample methodsFor: 'open tools' stamp: 'CamiloGomez 7/6/2012 17:37'!
roassalEasel

	"self new roassalEasel"
	
	ROEaselMorphic new visualize.
	! !

!VAExample methodsFor: 'layout' stamp: 'CamiloGomez 7/6/2012 12:20'!
simpleLayout

	"self new simpleLayout"

	"Not implemented yet"! !

!VAExample methodsFor: 'open tools' stamp: 'CamiloGomez 7/6/2012 15:55'!
transcript

	"self new transcript"
	
	Transcript visualize.! !

!VAExample methodsFor: 'va objects' stamp: 'CamiloGomez 7/17/2012 17:25'!
visualizeVADirector 

	"self new visualizeVADirector"

	| director view |
	
	view := ROView new.
	
	director := VADirector new name: 'Francis Ford Coppola'.
	
	director visualizeOn: view.	
	
	view open.! !

!VAExample methodsFor: 'va objects' stamp: 'CamiloGomez 7/24/2012 16:59'!
visualizeVAMovie

	"self new visualizeVAMovie"

	| movie movie2 movie3 movie4 movie5 view director |
	
	view := ROView new.
	movie := VAMovie new.
	movie2 := VAMovie new.
	movie3 := VAMovie new.
	movie4 := VAMovie new.
	movie5 := VAMovie new.
	director := (VADirector new name: 'Francis Ford Coppola'; directedMovie: movie; directedMovie: movie2; directedMovie: movie3 ; directedMovie: movie4; directedMovie: movie5 ).
	
	movie title: 'The Godfather'; 
		    director: director; 
		    duration: '175 minutes'; 
		    country: 'USA'; 
		    genre: 'Suspense'. 
	
	movie2 title: 'The Godfather 2';
			director: director;
			duration: '200 minutes';
			country: 'USA';
			genre: 'Drama'.
			
	movie3 title: 'The Godfather 3';
			director: director;
			duration: '163 minutes';
			country: 'USA';
			genre: 'Drama'.
			
	movie4 title: 'Apocalypse Now';
			director: director;
			duration: '153 minutes';
			country: 'USA';
			genre: 'Drama'.
			
	movie5 title: 'The Outsiders';
			director: director;
			duration: '91 minutes';
			country: 'USA';
			genre: 'Drama'.
	
	director visualizeOn: view.
	
	view open.! !

Object subclass: #VAMovie
	instanceVariableNames: 'title director duration country genre'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects'!

!VAMovie methodsFor: 'accesing' stamp: 'CamiloGomez 7/6/2012 18:05'!
country: aString

	country := aString! !

!VAMovie methodsFor: 'accesing' stamp: 'CamiloGomez 7/6/2012 18:04'!
director: aDirector

	director := aDirector! !

!VAMovie methodsFor: 'accesing' stamp: 'CamiloGomez 7/6/2012 16:25'!
duration: aTime

	duration := aTime! !

!VAMovie methodsFor: 'accesing' stamp: 'CamiloGomez 7/6/2012 18:05'!
genre: aString

	genre := aString! !

!VAMovie methodsFor: 'initialize-release' stamp: 'CamiloGomez 7/17/2012 17:53'!
initialize

	super initialize.
	
	title := ''.
	director := VADirector new.
	duration := ''.
	country := ''.
	genre := ''.! !

!VAMovie methodsFor: 'accesing' stamp: 'CamiloGomez 7/6/2012 18:05'!
title: aString

	title := aString! !

Object subclass: #VAPerson
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects'!

VAPerson subclass: #VADirector
	instanceVariableNames: 'directedMovies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects'!

!VADirector methodsFor: 'actions' stamp: 'CamiloGomez 7/6/2012 18:03'!
directedMovie: aMovie

	directedMovies add: aMovie.! !

!VADirector methodsFor: 'initialize-release' stamp: 'CamiloGomez 7/17/2012 17:50'!
initialize

	super initialize.
	directedMovies := OrderedCollection new.! !

!VADirector methodsFor: 'as yet unclassified' stamp: 'CamiloGomez 7/18/2012 15:40'!
shouldBeAddedAsElementOnEnter
	^ false! !

!VAPerson methodsFor: 'initialize-release' stamp: 'CamiloGomez 7/17/2012 17:27'!
initialize
	super initialize.
	
	name := ''.! !

!VAPerson methodsFor: 'actions' stamp: 'CamiloGomez 6/27/2012 15:25'!
name: aString
	name := aString! !

VAPerson subclass: #VAReader
	instanceVariableNames: 'readBooks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects'!

!VAReader methodsFor: 'initialize-release' stamp: 'CamiloGomez 7/6/2012 18:03'!
initialize

	super initialize.
	readBooks := OrderedCollection new.! !

!VAReader methodsFor: 'actions' stamp: 'CamiloGomez 7/6/2012 18:02'!
readBook: aBook
	readBooks add: aBook! !

!String methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 6/27/2012 15:39'!
shouldBeAddedAsElement
	^ false! !

!String methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/24/2012 16:54'!
shouldBeAddedAsElementOnEnter
	^ true! !

!Collection methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/24/2012 17:15'!
interactionFor: elementForSelf on: view

	(self size < 5)
	ifTrue: [super interactionFor: elementForSelf on: view]
	ifFalse: [
	| el view2 |
	
	view2 := ROView new.
	view2 title: 'VisuAule'.

	super interactionFor: elementForSelf on: view.
	elementForSelf @ (ROMenuActivable new item: 'visualize elements' action: [
			view2 add: elementForSelf.
			self do: [ :element | 
					element notNil
					ifTrue: [el := (elementForSelf model) roassalElementFor: element.
					(elementForSelf model) addElement: el of: elementForSelf on: view2.]].
			"self setThisLayout: ROSugiyamaLayout on: view2 elements."
			ROGridLayout on: ((view2 elements) select: [ :each | each model ~= elementForSelf model]).
			view2 open]).]! !

!Collection methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/21/2012 16:34'!
shouldBeAddedAsElement 
	^ true! !

!Collection methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/24/2012 16:51'!
shouldBeAddedAsElementOnEnter 
	^ false! !
